#!/bin/bash

####################################################################
# (C) Copyright 2012-2017 Hewlett Packard Enterprise Development LP.
# @(#) serviceguard-xdc UPCC Package Configuration Script.
# @(#) Product Name              :  Extended Distance Cluster on Linux
# @(#) Product Version           :  A.12.10.00
# @(#) Patch Name                :  
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of serviceguard-xdc.
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#
###################################################################

# Define Global Variables - commands that are used in this script
# or other scripts sourcing this file
typeset VMFS_MODULE="sg/vmfs"

# Cache in the cmviewcl output, Upon failure exit with failure
# status 1
typeset cmviewcl_retry_interval=3
typeset xdc_tmp_dir="/var/tmp"
typeset _cmviewcl_stderr=$xdc_tmp_dir/$log_prefix_"cmviewcl.err.$$"
cmviewcl_output=`$SGSBIN/cmviewcl -vf line -s config 2>$_cmviewcl_stderr`
if [ $? -ne 0 ]; then
    typeset -i _retry=3
    while [ $_retry -gt 0 ]; do
        sleep $cmviewcl_retry_interval
        cmviewcl_output=`$SGSBIN/cmviewcl -vf line -s config 2>$_cmviewcl_stderr`
        if [ $? -eq 0 ]; then
            break
        fi 
        (( _retry = _retry - 1 ))
        if [ $_retry -eq 0 ]; then
            sg_log 0 "Error: Failed to collect cluster configuration information"
            sg_log 0 "$(cat $_cmviewcl_stderr 2>/dev/null)"
            rm -f $_cmviewcl_stderr 2>/dev/null
            exit 1
        fi
    done
fi
rm -f $_cmviewcl_stderr 2>/dev/null

function activate_pr_on_single_disk
{
typeset DEVICE=$1
typeset -i num_vmdk=0

# If the node is a Physical Machine, PR activations must be 
# done. But if this is a Virtual Machine, then PR activations
# must be done if the package failover happened ungracefully 
# from a Physical Machine to 
# a VM.
echo "$cmviewcl_output" | grep -w "node:$(hostname)|virt_tech=vmware" >> /dev/null 2>&1
if [ $? -eq 0 ]; then
    # If there are VMFS files specified, skip PR activities
    sg_check_module_in_pkg $VMFS_MODULE
    if [ $? -eq 0 ]; then
        num_vmdk=${#SG_VMDK_FILE_NAME[*]}
        if [[ $num_vmdk -ge 1 ]] && ! sg_check_if_pr_module_required
        then
            return 0
        fi
    fi
fi

if [ "$PKG_PR_MODE" = "pr_enabled" ]
then
    ${SGCONF}/scripts/sg/pr_util.sh ${DEVICE} "activate"
    if [ $? -ne 0 ]
    then
        sg_log 0 "Activating PR failed on "${DEVICE}
        if sg_check_if_it_is_reconfiguration
        then
            sg_log 999 "ERROR: Activating PR failed on "${DEVICE}
            sg_set_reconfig_failure
            to_exit=1
        else
            exit 1
        fi
    fi
fi
return 0
}

function deactivate_pr_on_single_disk
{
typeset DEVICE=$1
typeset -i num_vmdk=0

# If the node is a Physical Machine, PR deactivation must be 
# done. But if this is a Virtual Machine, then PR deactivation
# must be done conditionally.
echo "$cmviewcl_output" | grep -w "node:$(hostname)|virt_tech=vmware" >> /dev/null 2>&1
if [ $? -eq 0 ]; then
    sg_check_module_in_pkg $VMFS_MODULE
    if [ $? -eq 0 ]; then
        num_vmdk=${#SG_VMDK_FILE_NAME[*]}
        # PR deactivation on a VM must be allowed only if the package 
        # failover happened ungracefully from a Physical Machine to 
        # a VM.
        if [[ $num_vmdk -ge 1 ]]
        then
            return 0
        fi
    fi
fi

if [ "$PKG_PR_MODE" = "pr_enabled" ]
then
    ${SGCONF}/scripts/sg/pr_util.sh ${DEVICE} "deactivate"
    if [ $? -ne 0 ]
    then
        sg_log 0 "Deactivating PR failed on "${DEVICE}
        if sg_check_if_it_is_reconfiguration
        then
            sg_log 999 "ERROR: Deactivating PR failed on "${DEVICE}
            sg_set_reconfig_failure
            to_exit=1
        else
            exit 1
        fi
    fi
fi
return 0
}

function lvm_mirror_check
{
typeset output="$1"
typeset myd="$2"
typeset pat="--- Logical volume ---"

echo "$output" | awk "/\<$(basename ${myd})\>/ {flag=1;print;next} /${pat}/{flag=0} flag {print}" | grep  -q "Mirrored volumes" >/dev/null
return $?
}

function build_lvm_list
{
   unset MIRRORED_VG_LIST
   for ((  I = 0 ;  I < ${#SG_VG[@]};  I++  ))
   do
       vgdisplay -v ${SG_VG[$I]} 2>&1 | grep -q "Mirrored volumes" >/dev/null
       if (( $? == 0 )); then
          if [ -z "$MIRRORED_VG_LIST" ]; then
             MIRRORED_VG_LIST=${SG_VG[I]}
          else
             MIRRORED_VG_LIST=$MIRRORED_VG_LIST" "${SG_VG[$I]}
          fi
       fi
   done
}

function find_if_pkg_has_mirrored_vg
{
typeset -i mirrored_vg_found=0
for ((  I = 0 ;  I < ${#SG_VG[@]};  I++  ))
do
     vgdisplay -v ${SG_VG[I]} 2>&1 | grep -q "Mirrored volumes" >/dev/null
     if (( $? == 0 )); then
        mirrored_vg_found=1
        break
     fi
done
return ${mirrored_vg_found}
}

function build_vxvm_dg_list
{
   unset MIRRORED_VXVM_DG_LIST
   for ((  I = 0 ;  I < ${#SG_VXVM_DG[@]};  I++  ))
   do
      site=$(vxdisk -s list | awk -v vg=${SG_VXVM_DG[I]} '
             /^site:/   { site = $2 }
             /^dgname:/ { if (vg == $2) { print site; exit } }')
      if [[ $site != "" ]] && [[ $site != "-" ]]; then
          if [ -z "$MIRRORED_VXVM_DG_LIST" ]; then
             MIRRORED_VXVM_DG_LIST=${SG_VXVM_DG[I]}
          else
             MIRRORED_VXVM_DG_LIST=$MIRRORED_VXVM_DG_LIST" "${SG_VXVM_DG[$I]}
          fi
      fi
   done
}

function find_if_pkg_has_mirrored_dg
{
   typeset -i mirrored_dg_found=0
   for ((  I = 0 ;  I < ${#SG_VXVM_DG[@]};  I++  ))
   do
      site=$(vxdisk -s list | awk -v vg=${SG_VXVM_DG[I]} '
             /^site:/   { site = $2 }
             /^dgname:/ { if (vg == $2) { print site; exit } }')
      if [[ $site != "" ]] && [[ $site != "-" ]]; then 
          mirrored_dg_found=1
          break
      fi
   done
return ${mirrored_dg_found}
}
